# variables that need to be passed from R:

# y = dependent
# X.fixed = fixed predictor matrix
# X.random = random predictor matrix

# n = number of data points
# P = number of fixed effects
# Q = number of random effects

# r1.cluster.i = cluster index for random effect 
# r1.cluster.n = number of clusters for random effect 1

# B.raw.hat is set to zero –> assuming means are estimated in fixed part of the model!


model {

	# model

	for (i in 1:n) {

		y[i] ~ dnorm(y.hat[i], y.tau)
		
		y.hat[i] <- fixed[i] + random[i]
		
		fixed[i] <- inprod(B.fixed[1:P],X.fixed[i,1:P])
		random[i] <- inprod(B.random[r1.cluster.i[i],1:Q],X.random[i,1:Q])
	}
		
		
	# error variance y	
	
	y.tau <- pow(y.noise, -2) 
	y.noise ~ dunif(0, 100)
	

	# random effects means/variance (means set to zero!)
	
	for (i in 1:Q) {
		
		xi[i] ~ dunif(0, 100)
		
		mu.raw[i] <- 0
		#mu.raw[i] ~ dnorm(0, .0001) 
				
		mu[i] <- xi[i] * mu.raw[i] 
	}
	
	
	# random effects variance matrix
	
	df <- Q+1
	Tau.B.raw[1:Q,1:Q] ~ dwish(W[,], df)
	Sigma.B.raw[1:Q,1:Q] <- inverse(Tau.B.raw[,]) 
	
	for (i in 1:Q) {

		sigma.B[i] <- xi[i] * sqrt(Sigma.B.raw[i,i]) 
	}	
	
	
	# random effects matrix
		
	for (j in 1:r1.cluster.n) { 
		
		B.raw[j,1:Q] ~ dmnorm(B.raw.hat[j,], Tau.B.raw[,]) 
		
		for(i in 1:Q) {
			
			B.raw.hat[j,i] <- mu.raw[i]
			B.random[j,i] <- xi[i] * B.raw[j,i] 
		}
	}
	
	
	# random effects correlation
		
	for (i in 1:Q) {
		
		for (j in 1:Q) {
			
			rho.B[i,j] <- Sigma.B.raw[i,j] / sqrt(Sigma.B.raw[i,i] * Sigma.B.raw[j,j]) 
		}		
	}	

	
	# priors for fixed effects
		
	for (f in 1:P) {   B.fixed[f] ~ dnorm(0, .0001)   }

	
}


