# variables that need to be passed from R:

# y = dependent
# X.fixed = fixed predictor matrix

# X.random1 = random predictor matrix #1
# X.random2 = random predictor matrix #2

# n = number of data points
# P = number of fixed effects
# Q1 = number of random effects #1
# Q2 = number of random effects #2

# r1.cluster.i = cluster index for random effect 
# r1.cluster.n = number of clusters for random effect 1

# r2.cluster.i = cluster index for random effect 
# r2.cluster.n = number of clusters for random effect 2


# B.raw.hat is set to zero –> assuming means are estimated in fixed part of the model!


model {

	# model

	for (i in 1:n) {

		y[i] ~ dnorm(y.hat[i], y.tau)
		
		y.hat[i] <- fixed[i] + random1[i] + random2[i]
		
		fixed[i] <- inprod(B.fixed[1:P],X.fixed[i,1:P])
		
		random1[i] <- inprod(B.random1[r1.cluster.i[i],1:Q1],X.random1[i,1:Q1])
		random2[i] <- inprod(B.random2[r2.cluster.i[i],1:Q2],X.random2[i,1:Q2])
		
	}
		
		
	# error variance y	
	
	y.tau <- pow(y.noise, -2) 
	y.noise ~ dunif(0, 100)
	

	# random effects #1 means/variance (means set to zero!)
	
	for (i in 1:Q1) {
		
		xi1[i] ~ dunif(0, 100)
		
		mu.raw1[i] <- 0
		#mu.raw1[i] ~ dnorm(0, .0001) 
				
		mu1[i] <- xi1[i] * mu.raw1[i] 
	}
	
	
	# random effects #1 variance matrix
	
	df1 <- Q1+1
	Tau.B.raw1[1:Q1,1:Q1] ~ dwish(W1[,], df1)
	Sigma.B.raw1[1:Q1,1:Q1] <- inverse(Tau.B.raw1[,]) 
	
	for (i in 1:Q1) {

		sigma.B1[i] <- xi1[i] * sqrt(Sigma.B.raw1[i,i]) 
	}	
	
	
	# random effects #1 matrix
		
	for (j in 1:r1.cluster.n) { 
		
		B.raw1[j,1:Q1] ~ dmnorm(B.raw.hat1[j,], Tau.B.raw1[,]) 
		
		for(i in 1:Q1) {
			
			B.raw.hat1[j,i] <- mu.raw1[i]
			B.random1[j,i] <- xi1[i] * B.raw1[j,i] 
		}
	}
	
	
	# random effects #1 correlation
		
	for (i in 1:Q1) {
		
		for (j in 1:Q1) {
			
			rho.B1[i,j] <- Sigma.B.raw1[i,j] / sqrt(Sigma.B.raw1[i,i] * Sigma.B.raw1[j,j]) 
		}		
	}	



	# random effects #2 means/variance (means set to zero!)
	
	for (i in 1:Q2) {
		
		xi2[i] ~ dunif(0, 100)
		
		mu.raw2[i] <- 0
		#mu.raw1[i] ~ dnorm(0, .0001) 
				
		mu2[i] <- xi2[i] * mu.raw2[i] 
	}
	
	
	# random effects #2 variance matrix
	
	df2 <- Q2+1
	Tau.B.raw2[1:Q2,1:Q2] ~ dwish(W2[,], df2)
	Sigma.B.raw2[1:Q2,1:Q2] <- inverse(Tau.B.raw2[,]) 
	
	for (i in 1:Q2) {

		sigma.B2[i] <- xi2[i] * sqrt(Sigma.B.raw2[i,i]) 
	}	
	
	
	# random effects #2 matrix
		
	for (j in 1:r2.cluster.n) { 
		
		B.raw2[j,1:Q2] ~ dmnorm(B.raw.hat2[j,], Tau.B.raw2[,]) 
		
		for(i in 1:Q2) {
			
			B.raw.hat2[j,i] <- mu.raw2[i]
			B.random2[j,i] <- xi2[i] * B.raw2[j,i] 
		}
	}
	
	
	# random effects #2 correlation
		
	for (i in 1:Q2) {
		
		for (j in 1:Q2) {
			
			rho.B2[i,j] <- Sigma.B.raw2[i,j] / sqrt(Sigma.B.raw2[i,i] * Sigma.B.raw2[j,j]) 
		}		
	}	

	
	
	# priors for fixed effects
		
	for (f in 1:P) {   B.fixed[f] ~ dnorm(0, .0001)   }

	
}



